@@ -16,7 +16,7 @@ use smithay_client_toolkit::{
     window::{Frame, FrameRequest, State, WindowState},
 };
 
-mod theme;
+pub mod theme;
 use theme::{ColorTheme, BORDER_SIZE, HEADER_SIZE};
 
 mod buttons;
@@ -24,16 +24,20 @@ use buttons::{ButtonKind, Buttons};
 
 use crate::theme::ColorMap;
 
+mod font;
+mod minitype;
 mod parts;
 mod pointer;
 mod surface;
 
+const ROBOTO_FONT: &[u8] = include_bytes!("../assets/Roboto-Regular.ttf");
+
 /*
  * Utilities
  */
 
 #[derive(Debug, Copy, Clone, PartialEq, Eq)]
-pub enum Location {
+enum Location {
     None,
     Head,
     Top,
@@ -51,7 +55,7 @@ pub enum Location {
  * The core frame
  */
 
-pub struct Inner {
+struct Inner {
     parts: Parts,
     size: (u32, u32),
     resizable: bool,
@@ -119,6 +123,24 @@ fn precise_location(buttons: &Buttons, old: Location, width: u32, x: f64, y: f64
     }
 }
 
+pub struct FrameConfig {
+    pub theme: ColorTheme,
+}
+
+impl FrameConfig {
+    pub fn light() -> Self {
+        Self {
+            theme: ColorTheme::light(),
+        }
+    }
+
+    pub fn dark() -> Self {
+        Self {
+            theme: ColorTheme::dark(),
+        }
+    }
+}
+
 /// A simple set of decorations
 #[derive(Debug)]
 pub struct AdwaitaFrame {
@@ -135,11 +157,13 @@ pub struct AdwaitaFrame {
 
     buttons: Rc<RefCell<Buttons>>,
     colors: ColorTheme,
+    font: font::Font,
+    title: Option<String>,
 }
 
 impl Frame for AdwaitaFrame {
     type Error = ::std::io::Error;
-    type Config = ();
+    type Config = FrameConfig;
     fn init(
         base_surface: &wl_surface::WlSurface,
         compositor: &Attached<wl_compositor::WlCompositor>,
@@ -182,6 +206,8 @@ impl Frame for AdwaitaFrame {
             surface_version: compositor.as_ref().version(),
             buttons: Default::default(),
             colors: Default::default(),
+            font: font::Font::from_bytes(ROBOTO_FONT).unwrap(),
+            title: None,
         })
     }
 
@@ -326,6 +352,8 @@ impl Frame for AdwaitaFrame {
                         inner.resizable,
                         self.active,
                         &self.colors,
+                        &self.font,
+                        self.title.as_ref(),
                         &self.buttons.borrow(),
                         &self
                             .pointers
@@ -553,9 +581,13 @@ impl Frame for AdwaitaFrame {
         }
     }
 
-    fn set_config(&mut self, _config: ()) {}
+    fn set_config(&mut self, config: FrameConfig) {
+        self.colors = config.theme;
+    }
 
-    fn set_title(&mut self, _title: String) {}
+    fn set_title(&mut self, title: String) {
+        self.title = Some(title);
+    }
 }
 
 impl Drop for AdwaitaFrame {
@@ -574,6 +606,8 @@ fn draw_headerbar(
     maximizable: bool,
     state: WindowState,
     colors: &ColorTheme,
+    font: &font::Font,
+    title: Option<&String>,
     buttons: &Buttons,
     mouses: &[Location],
 ) {
@@ -586,6 +620,47 @@ fn draw_headerbar(
 
     draw_headerbar_bg(pixmap, scale, margin_h, margin_v, colors);
 
+    if let Some(title) = title {
+        let size = 20.0 * scale as f64;
+
+        let canvas_w = pixmap.width() as f32;
+        let canvas_h = pixmap.height() as f32;
+
+        let header_w = canvas_w - margin_h * 2.0;
+        let header_h = canvas_h - margin_v;
+
+        let (w, h) = font.measure_text(title, size);
+        let text_w = w as f32;
+        let text_h = h as f32;
+
+        let x = header_w / 2.0 - text_w / 2.0;
+        let y = header_h / 2.0 - text_h / 2.0;
+
+        let x = margin_h + x;
+        let y = margin_v + y;
+
+        let (x, y) = if x + text_w < buttons.minimize.x() - 10.0 {
+            (x, y)
+        } else {
+            let y = header_h / 2.0 - text_h / 2.0;
+
+            let x = buttons.minimize.x() - text_w - 10.0;
+            let y = margin_v + y;
+            (x, y)
+        };
+
+        let x = x.max(margin_h + 5.0);
+
+        font.render_text(
+            size,
+            &colors.font_paint(),
+            pixmap,
+            (x as f64, y as f64),
+            title,
+            buttons.minimize.x() as f64 - 10.0,
+        );
+    }
+
     if buttons.close.x() > margin_h {
         buttons.close.draw_close(scale, colors, mouses, pixmap);
     }
